import { memo } from './utils';
export const observeElementRect = (instance, cb) => {
    const element = instance.scrollElement;
    if (!element) {
        return;
    }
    const targetWindow = instance.targetWindow;
    if (!targetWindow) {
        return;
    }
    const handler = (rect) => {
        const { width, height } = rect;
        cb({ width: Math.round(width), height: Math.round(height) });
    };
    handler(element.getBoundingClientRect());
    if (!targetWindow.ResizeObserver) {
        return () => { };
    }
    const observer = new targetWindow.ResizeObserver((entries) => {
        const entry = entries[0];
        if (entry?.borderBoxSize) {
            const box = entry.borderBoxSize[0];
            if (box) {
                handler({ width: box.inlineSize, height: box.blockSize });
                return;
            }
        }
        handler(element.getBoundingClientRect());
    });
    observer.observe(element, { box: 'border-box' });
    return () => {
        observer.unobserve(element);
    };
};
export class TreeVirtualizer {
    selectedId;
    visibleItems;
    expandedNodes;
    options;
    scrollElement = null;
    targetWindow = null;
    isScrolling = false;
    constructor(opts) {
        this.selectedId = '';
        this.visibleItems = [];
        this.expandedNodes = new Set();
        this.setOptions(opts);
        this.updateVisibleNodes();
    }
    setOptions = (opts) => {
        Object.entries(opts).forEach(([key, value]) => {
            if (typeof value === 'undefined')
                delete opts[key];
        });
        this.options = {
            debug: false,
            onChange: () => { },
            indexAttribute: 'data-index',
            enabled: true,
            ...opts
        };
    };
    notify = (sync) => {
        this.options.onChange?.(this, sync);
    };
    cleanup = () => {
        this.scrollElement = null;
        this.targetWindow = null;
    };
    _didMount = () => {
        return () => {
            this.cleanup();
        };
    };
    _willUpdate = () => {
        const scrollElement = this.options.enabled ? this.options.getScrollElement() : null;
        if (this.scrollElement !== scrollElement) {
            this.cleanup();
            this.scrollElement = scrollElement;
            if (this.scrollElement && 'ownerDocument' in this.scrollElement) {
                this.targetWindow = this.scrollElement.ownerDocument.defaultView;
            }
            else {
                this.targetWindow = this.scrollElement?.window ?? null;
            }
        }
    };
    getIndexes = memo(() => [this.visibleItems], (visibleItems) => {
        return visibleItems;
    }, {
        key: process.env.NODE_ENV !== 'production' && 'getIndexes',
        debug: () => this.options.debug
    });
    indexFromElement = (node) => {
        const attributeName = this.options.indexAttribute;
        const indexStr = node.getAttribute(attributeName);
        if (!indexStr) {
            console.warn(`Missing attribute name '${attributeName}={index}' on measured element.`);
            return -1;
        }
        return parseInt(indexStr, 10);
    };
    getVirtualTreeItems = memo(() => [this.visibleItems], (indexes) => {
        return indexes;
    }, {
        key: process.env.NODE_ENV !== 'production' && 'getVirtualTreeItems',
        debug: () => this.options.debug
    });
    /**
     * Sets the selected ID.
     *
     * @param id - The ID to be set as selected. Can be a string or a number.
     */
    setSelectedId(id) {
        this.selectedId = id;
        this.notify(true);
    }
    /**
     * Retrieves the currently selected ID.
     *
     * @returns {string | number} The ID of the selected item.
     */
    getSelectedId() {
        return this.selectedId;
    }
    // Set the intial visible nodes from the constructor
    updateVisibleNodes() {
        let nodesThatShouldBeVisible = this.options.data.filter((node) => {
            if (node.parentId === null) {
                return true;
            }
            return this.expandedNodes.has(node.parentId);
        });
        this.visibleItems = nodesThatShouldBeVisible;
    }
    /**
     * Adds or removes the node from the expandedNodes set
     * @param nodeId id of the node to toggle
     * @param index current index of the node
     * @returns
     */
    toggleNode(nodeId, index) {
        this.setSelectedId(nodeId);
        let node = this.options.data.find((node) => node.id === nodeId);
        if (!node) {
            return;
        }
        if (this.expandedNodes.has(node.id)) {
            this.expandedNodes.delete(node.id);
            if (node.children && node.children.length > 0) {
                const removeChildren = (children) => {
                    children.forEach((child) => {
                        const index = this.visibleItems.findIndex((node) => node.id === child.id);
                        if (index !== -1) {
                            this.expandedNodes.delete(child.id);
                            let visibleNodesCopy = [...this.visibleItems];
                            visibleNodesCopy.splice(index, 1);
                            this.visibleItems = visibleNodesCopy;
                            if (child.children && child.children.length > 0) {
                                removeChildren(child.children);
                            }
                        }
                    });
                };
                removeChildren(node.children);
            }
        }
        else {
            this.expandedNodes.add(node.id);
            if (node.children && node.children.length > 0) {
                let newVisibleItems = [...this.visibleItems];
                newVisibleItems = [
                    ...this.visibleItems.slice(0, index + 1),
                    ...node.children,
                    ...this.visibleItems.slice(index + 1)
                ];
                this.setVisibleNodes(newVisibleItems);
            }
            else {
                this.setVisibleNodes([...this.visibleItems]);
            }
        }
    }
    setVisibleNodes(visibleNodes) {
        this.visibleItems = visibleNodes;
        this.notify(true);
    }
}
